@@ -21,6 +21,7 @@ __all__ = ['Integer', 'Boolean', 'BitString', 'OctetString', 'Null',
 
 # "Simple" ASN.1 types (yet incomplete)
 
+
 class Integer(base.AbstractSimpleAsn1Item):
     """Create |ASN.1| type or object.
 
@@ -45,14 +46,14 @@ class Integer(base.AbstractSimpleAsn1Item):
     : :py:class:`pyasn1.error.PyAsn1Error`
         On constraint violation or bad initializer.
     """
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x02)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -167,7 +168,8 @@ class Integer(base.AbstractSimpleAsn1Item):
         return int(self._value)
 
     if sys.version_info[0] <= 2:
-        def __long__(self): return long(self._value)
+        def __long__(self):
+            return long(self._value)
 
     def __float__(self):
         return float(self._value)
@@ -317,14 +319,14 @@ class Integer(base.AbstractSimpleAsn1Item):
 class Boolean(Integer):
     __doc__ = Integer.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x01),
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = Integer.subtypeSpec + constraint.SingleValueConstraint(0, 1)
@@ -371,14 +373,14 @@ class BitString(base.AbstractSimpleAsn1Item):
     : :py:class:`pyasn1.error.PyAsn1Error`
         On constraint violation or bad initializer.
     """
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x03)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -482,7 +484,7 @@ class BitString(base.AbstractSimpleAsn1Item):
 
         Parameters
         ----------
-        value : :class:`int`, :class:`str` or |ASN.1| object
+        value: :class:`int`, :class:`str` or |ASN.1| object
             Initialization value to pass to new ASN.1 object instead of
             inheriting one from the caller.
 
@@ -632,10 +634,17 @@ class BitString(base.AbstractSimpleAsn1Item):
         """Get |ASN.1| value as a text string of bits.
         """
         binString = binary.bin(self._value)[2:]
-        return '0'*(len(self._value) - len(binString)) + binString
+        return '0' * (len(self._value) - len(binString)) + binString
 
     @classmethod
     def fromHexString(cls, value):
+        """Create a |ASN.1| object initialized from the hex string.
+        
+        Parameters
+        ----------
+        value: :class:`str`
+            Text string like 'DEADBEEF'
+        """
         try:
             return cls.SizedInteger(value, 16).setBitLength(len(value) * 4)
 
@@ -644,6 +653,13 @@ class BitString(base.AbstractSimpleAsn1Item):
 
     @classmethod
     def fromBinaryString(cls, value):
+        """Create a |ASN.1| object initialized from a string of '0' and '1'.
+
+        Parameters
+        ----------
+        value: :class:`str`
+            Text string like '1010111'
+        """
         try:
             return cls.SizedInteger(value or '0', 2).setBitLength(len(value))
 
@@ -652,6 +668,13 @@ class BitString(base.AbstractSimpleAsn1Item):
 
     @classmethod
     def fromOctetString(cls, value, padding=0):
+        """Create a |ASN.1| object initialized from a string.
+
+        Parameters
+        ----------
+        value: :class:`str` (Py2) or :class:`bytes` (Py3)
+            Text string like '\\\\x01\\\\xff' (Py2) or b'\\\\x01\\\\xff' (Py3)
+        """
         return cls(cls.SizedInteger(integer.from_bytes(value) >> padding).setBitLength(len(value) * 8 - padding))
 
     def prettyIn(self, value):
@@ -764,14 +787,14 @@ class OctetString(base.AbstractSimpleAsn1Item):
     : :py:class:`pyasn1.error.PyAsn1Error`
         On constraint violation or bad initializer.
     """
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x04)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -807,8 +830,6 @@ class OctetString(base.AbstractSimpleAsn1Item):
         if 'encoding' not in kwargs:
             kwargs['encoding'] = self.encoding
 
-        self.__asNumbersCache = None
-
         base.AbstractSimpleAsn1Item.__init__(self, value, **kwargs)
 
     def clone(self, value=noValue, **kwargs):
@@ -900,7 +921,7 @@ class OctetString(base.AbstractSimpleAsn1Item):
                     return value.encode(self.encoding)
                 except (LookupError, UnicodeEncodeError):
                     raise error.PyAsn1Error(
-                        'Can\'t encode string \'%s\' with \'%s\' codec' % (value, self.encoding)
+                        "Can't encode string '%s' with codec %s" % (value, self.encoding)
                     )
             elif isinstance(value, (tuple, list)):
                 try:
@@ -921,16 +942,14 @@ class OctetString(base.AbstractSimpleAsn1Item):
 
             except UnicodeDecodeError:
                 raise error.PyAsn1Error(
-                    'Can\'t decode string \'%s\' with \'%s\' codec' % (self._value, self.encoding)
+                    "Can't decode string '%s' with codec %s" % (self._value, self.encoding)
                 )
 
         def asOctets(self):
             return str(self._value)
 
         def asNumbers(self):
-            if self.__asNumbersCache is None:
-                self.__asNumbersCache = tuple([ord(x) for x in self._value])
-            return self.__asNumbersCache
+            return tuple([ord(x) for x in self._value])
 
     else:
         def prettyIn(self, value):
@@ -968,9 +987,7 @@ class OctetString(base.AbstractSimpleAsn1Item):
             return bytes(self._value)
 
         def asNumbers(self):
-            if self.__asNumbersCache is None:
-                self.__asNumbersCache = tuple(self._value)
-            return self.__asNumbersCache
+            return tuple(self._value)
 
     def prettyOut(self, value):
         if sys.version_info[0] <= 2:
@@ -986,12 +1003,18 @@ class OctetString(base.AbstractSimpleAsn1Item):
 
             except UnicodeDecodeError:
                 raise error.PyAsn1Error(
-                    'Can\'t decode string \'%s\' with \'%s\' codec at \'%s\'' % (
-                    value, self.encoding, self.__class__.__name__)
+                    "Can't decode string '%s' with '%s' codec at '%s'" % (value, self.encoding, self.__class__.__name__)
                 )
 
     @staticmethod
     def fromBinaryString(value):
+        """Create a |ASN.1| object initialized from a string of '0' and '1'.
+
+        Parameters
+        ----------
+        value: :class:`str`
+            Text string like '1010111'
+        """
         bitNo = 8
         byte = 0
         r = []
@@ -1016,6 +1039,13 @@ class OctetString(base.AbstractSimpleAsn1Item):
 
     @staticmethod
     def fromHexString(value):
+        """Create a |ASN.1| object initialized from the hex string.
+
+        Parameters
+        ----------
+        value: :class:`str`
+            Text string like 'DEADBEEF'
+        """
         r = []
         p = []
         for v in value:
@@ -1052,9 +1082,7 @@ class OctetString(base.AbstractSimpleAsn1Item):
     # Immutable sequence object protocol
 
     def __len__(self):
-        if self._len is None:
-            self._len = len(self._value)
-        return self._len
+        return len(self._value)
 
     def __getitem__(self, i):
         if i.__class__ is slice:
@@ -1110,7 +1138,7 @@ class Null(OctetString):
     """
     defaultValue = ''.encode()  # This is tightly constrained
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
@@ -1202,14 +1230,14 @@ class ObjectIdentifier(base.AbstractSimpleAsn1Item):
     : :py:class:`pyasn1.error.PyAsn1Error`
         On constraint violation or bad initializer.
     """
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x06)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -1229,9 +1257,7 @@ class ObjectIdentifier(base.AbstractSimpleAsn1Item):
     # Sequence object protocol
 
     def __len__(self):
-        if self._len is None:
-            self._len = len(self._value)
-        return self._len
+        return len(self._value)
 
     def __getitem__(self, i):
         if i.__class__ is slice:
@@ -1342,14 +1368,14 @@ class Real(base.AbstractSimpleAsn1Item):
         _plusInf = _minusInf = None
         _inf = ()
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x09)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -1559,7 +1585,8 @@ class Real(base.AbstractSimpleAsn1Item):
         return int(float(self))
 
     if sys.version_info[0] <= 2:
-        def __long__(self): return long(float(self))
+        def __long__(self):
+            return long(float(self))
 
     def __float__(self):
         if self._value in self._inf:
@@ -1592,7 +1619,8 @@ class Real(base.AbstractSimpleAsn1Item):
         return self.clone(math.ceil(float(self)))
 
     if sys.version_info[0:2] > (2, 5):
-        def __trunc__(self): return self.clone(math.trunc(float(self)))
+        def __trunc__(self):
+            return self.clone(math.trunc(float(self)))
 
     def __lt__(self, value):
         return float(self) < value
@@ -1642,14 +1670,14 @@ class Real(base.AbstractSimpleAsn1Item):
 class Enumerated(Integer):
     __doc__ = Integer.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatSimple, 0x0A)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -1684,6 +1712,17 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
         Object representing collection size constraint
      """
 
+    def __init__(self, *args, **kwargs):
+        # support positional params for backward compatibility
+        if args:
+            for key, value in zip(('componentType', 'tagSet',
+                                   'subtypeSpec', 'sizeSpec'), args):
+                if key in kwargs:
+                    raise error.PyAsn1Error('Conflicting positional and keyword params!')
+                kwargs['componentType'] = value
+
+        base.AbstractConstructedAsn1Item.__init__(self, **kwargs)
+
     # Python list protocol
 
     def clear(self):
@@ -1715,7 +1754,7 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
 
     def _cloneComponentValues(self, myClone, cloneValueFlag):
         for idx, componentValue in enumerate(self._componentValues):
-            if componentValue is not None:
+            if componentValue is not noValue:
                 if isinstance(componentValue, base.AbstractConstructedAsn1Item):
                     myClone.setComponentByPosition(
                         idx, componentValue.clone(cloneValueFlag=cloneValueFlag)
@@ -1732,7 +1771,7 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
         ----------
         idx : :class:`int`
             Component index (zero-based). Must either refer to an existing
-            component or to N+1 component (of *componentType is set). In the latter
+            component or to N+1 component (if *componentType* is set). In the latter
             case a new component type gets instantiated and appended to the |ASN.1|
             sequence.
 
@@ -1786,28 +1825,31 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
         IndexError:
             When idx > len(self)
         """
+        if value is None:  # backward compatibility
+            value = noValue
+
         componentType = self.componentType
 
         try:
             currentValue = self._componentValues[idx]
         except IndexError:
-            currentValue = None
+            currentValue = noValue
 
             if len(self._componentValues) < idx:
                 raise error.PyAsn1Error('Component index out of range')
 
-        if value is noValue or value is None:
+        if value is noValue:
             if componentType is not None:
                 value = componentType.clone()
-            elif currentValue is None:
+            elif currentValue is noValue:
                 raise error.PyAsn1Error('Component type not defined')
         elif not isinstance(value, base.Asn1Item):
             if componentType is not None and isinstance(componentType, base.AbstractSimpleAsn1Item):
                 value = componentType.clone(value=value)
-            elif currentValue is not None and isinstance(currentValue, base.AbstractSimpleAsn1Item):
+            elif currentValue is not noValue and isinstance(currentValue, base.AbstractSimpleAsn1Item):
                 value = currentValue.clone(value=value)
             else:
-                raise error.PyAsn1Error('%s undefined component type' % componentType.__class__.__name__)
+                raise error.PyAsn1Error('Non-ASN.1 value %r and undefined component type at %r' % (value, self))
         elif componentType is not None:
             if self.strictConstraints:
                 if not componentType.isSameTypeWith(value, matchTags, matchConstraints):
@@ -1824,7 +1866,7 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
                 exType, exValue, exTb = sys.exc_info()
                 raise exType('%s at %s' % (exValue, self.__class__.__name__))
 
-        if currentValue is None:
+        if currentValue is noValue:
             self._componentValues.append(value)
         else:
             self._componentValues[idx] = value
@@ -1833,49 +1875,56 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
 
     @property
     def componentTagMap(self):
-        if self.componentType.asn1Object is not None:
-            return self.componentType.asn1Object.tagMap
+        if self.componentType is not None:
+            return self.componentType.tagMap
 
     def prettyPrint(self, scope=0):
         scope += 1
         representation = self.__class__.__name__ + ':\n'
-        for idx in range(len(self._componentValues)):
+        for idx, componentValue in enumerate(self._componentValues):
             representation += ' ' * scope
-            if self._componentValues[idx] is None:
+            if (componentValue is noValue and
+                    self.componentType is not None):
                 representation += '<empty>'
             else:
-                representation += self._componentValues[idx].prettyPrint(scope)
+                representation += componentValue.prettyPrint(scope)
         return representation
 
     def prettyPrintType(self, scope=0):
         scope += 1
         representation = '%s -> %s {\n' % (self.tagSet, self.__class__.__name__)
-        if self.componentType.asn1Object is not None:
+        if self.componentType is not None:
             representation += ' ' * scope
-            representation += self.componentType.asn1Object.prettyPrintType(scope)
+            representation += self.componentType.prettyPrintType(scope)
         return representation + '\n' + ' ' * (scope - 1) + '}'
 
 
     @property
     def isValue(self):
-        """Indicate if |ASN.1| object components represent ASN.1 type or ASN.1 value.
+        """Indicate if |ASN.1| object represents ASN.1 type or ASN.1 value.
 
-        The PyASN1 type objects can only participate in types comparison
-        and serve as a blueprint for serialization codecs to resolve
-        ambiguous types.
+        In other words, if *isValue* is `True`, then the ASN.1 object is
+        initialized.
 
-        The PyASN1 value objects can additionally participate in most
-        of built-in Python operations.
+        For the purpose of this check, empty |ASN.1| object is considered
+        as initialized.
 
         Returns
         -------
         : :class:`bool`
-            :class:`True` if all |ASN.1| components represent value and type,
-            :class:`False` if at least one |ASN.1| component represents just ASN.1 type.
-        """
-        if not self._componentValues:
-            return False
+            :class:`True` if object represents ASN.1 value and type,
+            :class:`False` if object represents just ASN.1 type.
 
+        Note
+        ----
+        There is an important distinction between PyASN1 type and value objects.
+        The PyASN1 type objects can only participate in ASN.1 type
+        operations (subtyping, comparison etc) and serve as a
+        blueprint for serialization codecs to resolve ambiguous types.
+
+        The PyASN1 value objects can additionally participate in most
+        of built-in Python operations.
+        """
         for componentValue in self._componentValues:
             if not componentValue.isValue:
                 return False
@@ -1886,7 +1935,7 @@ class SequenceOfAndSetOfBase(base.AbstractConstructedAsn1Item):
 class SequenceOf(SequenceOfAndSetOfBase):
     __doc__ = SequenceOfAndSetOfBase.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
@@ -1897,7 +1946,7 @@ class SequenceOf(SequenceOfAndSetOfBase):
     #: object representing ASN.1 type allowed within |ASN.1| type
     componentType = None
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -1913,7 +1962,7 @@ class SequenceOf(SequenceOfAndSetOfBase):
 class SetOf(SequenceOfAndSetOfBase):
     __doc__ = SequenceOfAndSetOfBase.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
@@ -1924,7 +1973,7 @@ class SetOf(SequenceOfAndSetOfBase):
     #: object representing ASN.1 type allowed within |ASN.1| type
     componentType = None
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -1960,9 +2009,52 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
     #: object representing named ASN.1 types allowed within |ASN.1| type
     componentType = namedtype.NamedTypes()
 
+
+    class DynamicNames(object):
+        """Fields names/positions mapping for component-less objects"""
+        def __init__(self):
+            self._keyToIdxMap = {}
+            self._idxToKeyMap = {}
+
+        def __len__(self):
+            return len(self._keyToIdxMap)
+
+        def __contains__(self, item):
+            return item in self._keyToIdxMap or item in self._idxToKeyMap
+
+        def __iter__(self):
+            return (self._idxToKeyMap[idx] for idx in range(len(self._idxToKeyMap)))
+
+        def __getitem__(self, item):
+            try:
+                return self._keyToIdxMap[item]
+
+            except KeyError:
+                return self._idxToKeyMap[item]
+
+        def getNameByPosition(self, idx):
+            try:
+                return self._idxToKeyMap[idx]
+
+            except KeyError:
+                raise error.PyAsn1Error('Type position out of range')
+
+        def getPositionByName(self, name):
+            try:
+                return self._keyToIdxMap[name]
+
+            except KeyError:
+                raise error.PyAsn1Error('Name %s not found' % (name,))
+
+        def addField(self, idx):
+            self._keyToIdxMap['field-%d' % idx] = idx
+            self._idxToKeyMap[idx] = 'field-%d' % idx
+
+
     def __init__(self, **kwargs):
         base.AbstractConstructedAsn1Item.__init__(self, **kwargs)
         self._componentTypeLen = len(self.componentType)
+        self._dynamicNames = self._componentTypeLen or self.DynamicNames()
 
     def __getitem__(self, idx):
         if octets.isStringType(idx):
@@ -1977,23 +2069,29 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
             base.AbstractConstructedAsn1Item.__setitem__(self, idx, value)
 
     def __contains__(self, key):
-        return key in self.componentType
+        if self._componentTypeLen:
+            return key in self.componentType
+        else:
+            return key in self._dynamicNames
 
     def __iter__(self):
-        return iter(self.componentType)
+        return iter(self.componentType or self._dynamicNames)
 
     # Python dict protocol
 
     def values(self):
-        for idx in range(self._componentTypeLen):
+        for idx in range(self._componentTypeLen or len(self._dynamicNames)):
             yield self[idx]
 
     def keys(self):
-        return iter(self.componentType)
+        return iter(self)
 
     def items(self):
-        for idx in range(self._componentTypeLen):
-            yield self.componentType[idx].getName(), self[idx]
+        for idx in range(self._componentTypeLen or len(self._dynamicNames)):
+            if self._componentTypeLen:
+                yield self.componentType[idx].name, self[idx]
+            else:
+                yield self._dynamicNames[idx], self[idx]
 
     def update(self, *iterValue, **mappingValue):
         for k, v in iterValue:
@@ -2003,10 +2101,11 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
 
     def clear(self):
         self._componentValues = []
+        self._dynamicNames = self.DynamicNames()
 
     def _cloneComponentValues(self, myClone, cloneValueFlag):
         for idx, componentValue in enumerate(self._componentValues):
-            if componentValue is not None:
+            if componentValue is not noValue:
                 if isinstance(componentValue, base.AbstractConstructedAsn1Item):
                     myClone.setComponentByPosition(
                         idx, componentValue.clone(cloneValueFlag=cloneValueFlag)
@@ -2029,9 +2128,16 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
         : :py:class:`~pyasn1.type.base.PyAsn1Item`
             Instantiate |ASN.1| component type or return existing component value
         """
-        return self.getComponentByPosition(
-            self.componentType.getPositionByName(name)
-        )
+        if self._componentTypeLen:
+            idx = self.componentType.getPositionByName(name)
+        else:
+            try:
+                idx = self._dynamicNames.getPositionByName(name)
+
+            except KeyError:
+                raise error.PyAsn1Error('Name %s not found' % (name,))
+
+        return self.getComponentByPosition(idx)
 
     def setComponentByName(self, name, value=noValue,
                            verifyConstraints=True,
@@ -2063,8 +2169,17 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
         -------
         self
         """
+        if self._componentTypeLen:
+            idx = self.componentType.getPositionByName(name)
+        else:
+            try:
+                idx = self._dynamicNames.getPositionByName(name)
+
+            except KeyError:
+                raise error.PyAsn1Error('Name %s not found' % (name,))
+
         return self.setComponentByPosition(
-            self.componentType.getPositionByName(name), value, verifyConstraints, matchTags, matchConstraints
+            idx, value, verifyConstraints, matchTags, matchConstraints
         )
 
     def getComponentByPosition(self, idx):
@@ -2087,9 +2202,9 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
         try:
             componentValue = self._componentValues[idx]
         except IndexError:
-            componentValue = None
+            componentValue = noValue
 
-        if componentValue is None:
+        if componentValue is noValue:
             self.setComponentByPosition(idx)
 
         return self._componentValues[idx]
@@ -2127,22 +2242,25 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
         -------
         self
         """
+        if value is None:  # backward compatibility
+            value = noValue
+
         componentType = self.componentType
         componentTypeLen = self._componentTypeLen
 
         try:
             currentValue = self._componentValues[idx]
         except IndexError:
-            currentValue = None
+            currentValue = noValue
             if componentTypeLen:
                 if componentTypeLen < idx:
                     raise IndexError('component index out of range')
-                self._componentValues = [None] * componentTypeLen
+                self._componentValues = [noValue] * componentTypeLen
 
-        if value is None or value is noValue:
+        if value is noValue:
             if componentTypeLen:
                 value = componentType.getTypeByPosition(idx).clone()
-            elif currentValue is None:
+            elif currentValue is noValue:
                 raise error.PyAsn1Error('Component type not defined')
         elif not isinstance(value, base.Asn1Item):
             if componentTypeLen:
@@ -2151,13 +2269,13 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
                     value = subComponentType.clone(value=value)
                 else:
                     raise error.PyAsn1Error('%s can cast only scalar values' % componentType.__class__.__name__)
-            elif currentValue is not None and isinstance(currentValue, base.AbstractSimpleAsn1Item):
+            elif currentValue is not noValue and isinstance(currentValue, base.AbstractSimpleAsn1Item):
                 value = currentValue.clone(value=value)
             else:
                 raise error.PyAsn1Error('%s undefined component type' % componentType.__class__.__name__)
         elif (matchTags or matchConstraints) and componentTypeLen:
             subComponentType = componentType.getTypeByPosition(idx)
-            if subComponentType is not None:
+            if subComponentType is not noValue:
                 if self.strictConstraints:
                     if not subComponentType.isSameTypeWith(value, matchTags, matchConstraints):
                         raise error.PyAsn1Error('Component value is tag-incompatible: %r vs %r' % (value, componentType))
@@ -2173,35 +2291,41 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
                 exType, exValue, exTb = sys.exc_info()
                 raise exType('%s at %s' % (exValue, self.__class__.__name__))
 
-        if componentTypeLen:
+        if componentTypeLen or idx in self._dynamicNames:
             self._componentValues[idx] = value
         elif len(self._componentValues) == idx:
             self._componentValues.append(value)
+            self._dynamicNames.addField(idx)
         else:
             raise error.PyAsn1Error('Component index out of range')
 
         return self
 
-    def getNameByPosition(self, idx):
-        if self._componentTypeLen:
-            return self.componentType.getNameByPosition(idx)
-
     @property
     def isValue(self):
-        """Indicate if |ASN.1| object components represent ASN.1 type or ASN.1 value.
+        """Indicate if |ASN.1| object represents ASN.1 type or ASN.1 value.
 
-        The PyASN1 type objects can only participate in types comparison
-        and serve as a blueprint for serialization codecs to resolve
-        ambiguous types.
+        In other words, if *isValue* is `True`, then the ASN.1 object is
+        initialized.
 
-        The PyASN1 value objects can additionally participate in most
-        of built-in Python operations.
+        For the purpose of check, the *OPTIONAL* and *DEFAULT* fields are
+        unconditionally considered as initialized.
 
         Returns
         -------
         : :class:`bool`
-            :class:`True` if all |ASN.1| components represent value and type,
-            :class:`False` if at least one |ASN.1| component represents just ASN.1 type.
+            :class:`True` if object represents ASN.1 value and type,
+            :class:`False` if object represents just ASN.1 type.
+
+        Note
+        ----
+        There is an important distinction between PyASN1 type and value objects.
+        The PyASN1 type objects can only participate in ASN.1 type
+        operations (subtyping, comparison etc) and serve as a
+        blueprint for serialization codecs to resolve ambiguous types.
+
+        The PyASN1 value objects can additionally participate in most
+        of built-in Python operations.
         """
         componentType = self.componentType
 
@@ -2209,7 +2333,8 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
             for idx, subComponentType in enumerate(componentType.namedTypes):
                 if subComponentType.isDefaulted or subComponentType.isOptional:
                     continue
-                if not self._componentValues or self._componentValues[idx] is None or not self._componentValues[idx].isValue:
+                if (not self._componentValues or
+                        not self._componentValues[idx].isValue):
                     return False
 
         else:
@@ -2229,23 +2354,29 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
         """
         scope += 1
         representation = self.__class__.__name__ + ':\n'
-        for idx in range(len(self._componentValues)):
-            if self._componentValues[idx] is not None:
+        for idx, componentValue in enumerate(self._componentValues):
+            if componentValue is not noValue:
                 representation += ' ' * scope
-                representation += self.componentType.getNameByPosition(idx)
+                if self.componentType:
+                    representation += self.componentType.getNameByPosition(idx)
+                else:
+                    representation += self._dynamicNames.getNameByPosition(idx)
                 representation = '%s=%s\n' % (
-                    representation, self._componentValues[idx].prettyPrint(scope)
+                    representation, componentValue.prettyPrint(scope)
                 )
         return representation
 
     def prettyPrintType(self, scope=0):
         scope += 1
         representation = '%s -> %s {\n' % (self.tagSet, self.__class__.__name__)
-        for idx in range(len(self.componentType)):
+        for idx, componentType in enumerate(self.componentType.values() or self._componentValues):
             representation += ' ' * scope
-            representation += '"%s"' % self.componentType.getNameByPosition(idx)
+            if self.componentType:
+                representation += '"%s"' % self.componentType.getNameByPosition(idx)
+            else:
+                representation += '"%s"' % self._dynamicNames.getNameByPosition(idx)
             representation = '%s = %s\n' % (
-                representation, self.componentType.getTypeByPosition(idx).prettyPrintType(scope)
+                representation, componentType.prettyPrintType(scope)
             )
         return representation + '\n' + ' ' * (scope - 1) + '}'
 
@@ -2258,18 +2389,22 @@ class SequenceAndSetBase(base.AbstractConstructedAsn1Item):
         if self._componentTypeLen:
             return self.componentType
 
+    def getNameByPosition(self, idx):
+        if self._componentTypeLen:
+            return self.componentType[idx].name
+
 
 class Sequence(SequenceAndSetBase):
     __doc__ = SequenceAndSetBase.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
         tag.Tag(tag.tagClassUniversal, tag.tagFormatConstructed, 0x10)
     )
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -2285,6 +2420,8 @@ class Sequence(SequenceAndSetBase):
     # Disambiguation ASN.1 types identification
     typeId = SequenceAndSetBase.getTypeId()
 
+    # backward compatibility
+
     def getComponentTagMapNearPosition(self, idx):
         if self.componentType:
             return self.componentType.getTagMapNearPosition(idx)
@@ -2299,7 +2436,7 @@ class Sequence(SequenceAndSetBase):
 class Set(SequenceAndSetBase):
     __doc__ = SequenceAndSetBase.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.initTagSet(
@@ -2310,7 +2447,7 @@ class Set(SequenceAndSetBase):
     #: object representing ASN.1 type allowed within |ASN.1| type
     componentType = namedtype.NamedTypes()
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -2410,7 +2547,7 @@ class Set(SequenceAndSetBase):
 class Choice(Set):
     __doc__ = Set.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.TagSet()  # untagged
@@ -2419,7 +2556,7 @@ class Choice(Set):
     #: object representing ASN.1 type allowed within |ASN.1| type
     componentType = namedtype.NamedTypes()
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
@@ -2660,12 +2797,12 @@ class Choice(Set):
 class Any(OctetString):
     __doc__ = OctetString.__doc__
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.tag.TagSet` object representing ASN.1 tag(s)
     #: associated with |ASN.1| type.
     tagSet = tag.TagSet()  # untagged
 
-    #: Set (class attribute) or return (class or instance attribute) a
+    #: Set (on class, not on instance) or return a
     #: :py:class:`~pyasn1.type.constraint.ConstraintsIntersection` object
     #: imposing constraints on |ASN.1| type initialization values.
     subtypeSpec = constraint.ConstraintsIntersection()
